@@ -3,7 +3,7 @@
 //
 // THIS MODULE IS PUBLICLY LICENSED
 //
-// Copyright 2001-2018 by Wilson Snyder.  This program is free software;
+// Copyright 2001-2019 by Wilson Snyder.  This program is free software;
 // you can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 //
@@ -25,12 +25,13 @@
 
 #include "verilatedos.h"
 #include "verilated.h"
+
 #include "gtkwave/fstapi.h"
 
-#include <string>
-#include <vector>
 #include <list>
 #include <map>
+#include <string>
+#include <vector>
 
 class VerilatedFst;
 class VerilatedFstCallInfo;
@@ -43,6 +44,7 @@ typedef void (*VerilatedFstCallback_t)(VerilatedFst* vcdp, void* userthis, vluin
 
 class VerilatedFst {
     typedef std::map<vluint32_t, fstHandle> Code2SymbolType;
+    typedef std::map<int, fstEnumHandle> Local2FstDtype;
     typedef std::vector<VerilatedFstCallInfo*> CallbackVec;
 private:
     void* m_fst;
@@ -52,11 +54,12 @@ private:
     std::string m_module;
     CallbackVec m_callbacks;  ///< Routines to perform dumping
     Code2SymbolType m_code2symbol;
+    Local2FstDtype m_local2fstdtype;
     std::list<std::string> m_curScope;
     // CONSTRUCTORS
     VL_UNCOPYABLE(VerilatedFst);
     void declSymbol(vluint32_t code, const char* name,
-                    fstVarDir vardir, fstVarType vartype,
+                    int dtypenum, fstVarDir vardir, fstVarType vartype,
                     int arraynum, vluint32_t len);
     // helpers
     std::vector<char> m_valueStrBuffer;
@@ -74,11 +77,11 @@ public:
         fstWriterClose(m_fst);
         m_fst = NULL;
     }
-    // void set_time_unit(const char* unit); ///< Set time units (s/ms, defaults to ns)
-    // void set_time_unit(const std::string& unit) { set_time_unit(unit.c_str()); }
+    void set_time_unit(const char* unitp) { fstWriterSetTimescaleFromString(m_fst, unitp); }
+    void set_time_unit(const std::string& unit) { set_time_unit(unit.c_str()); }
 
-    // void set_time_resolution(const char* unit); ///< Set time resolution (s/ms, defaults to ns)
-    // void set_time_resolution(const std::string& unit) { set_time_resolution(unit.c_str()); }
+    void set_time_resolution(const char* unitp) { if (unitp) {} }
+    void set_time_resolution(const std::string& unit) { set_time_resolution(unit.c_str()); }
 
     // double timescaleToDouble(const char* unitp);
     // std::string doubleToTimescale(double value);
@@ -90,36 +93,46 @@ public:
     /// Inside dumping routines, called each cycle to make the dump
     void dump(vluint64_t timeui);
     /// Inside dumping routines, declare callbacks for tracings
-    void addCallback(VerilatedFstCallback_t init, VerilatedFstCallback_t full,
-                     VerilatedFstCallback_t change,
+    void addCallback(VerilatedFstCallback_t initcb, VerilatedFstCallback_t fullcb,
+                     VerilatedFstCallback_t changecb,
                      void* userthis) VL_MT_UNSAFE_ONE;
 
     /// Inside dumping routines, declare a module
     void module(const std::string& name);
+    /// Inside dumping routines, declare a data type
+    void declDTypeEnum(int dtypenum, const char* name, vluint32_t elements,
+                       unsigned int minValbits,
+                       const char** itemNamesp, const char** itemValuesp);
     /// Inside dumping routines, declare a signal
-    void declBit(vluint32_t code, const char* name, fstVarDir vardir, fstVarType vartype,
+    void declBit(vluint32_t code, const char* name,
+                 int dtypenum, fstVarDir vardir, fstVarType vartype,
                  int arraynum) {
-        declSymbol(code, name, vardir, vartype, arraynum, 1);
+        declSymbol(code, name, dtypenum, vardir, vartype, arraynum, 1);
     }
-    void declBus(vluint32_t code, const char* name, fstVarDir vardir, fstVarType vartype,
+    void declBus(vluint32_t code, const char* name,
+                 int dtypenum, fstVarDir vardir, fstVarType vartype,
                  int arraynum, int msb, int lsb) {
-        declSymbol(code, name, vardir, vartype, arraynum, msb - lsb + 1);
+        declSymbol(code, name, dtypenum, vardir, vartype, arraynum, msb - lsb + 1);
     }
-    void declDouble(vluint32_t code, const char* name, fstVarDir vardir, fstVarType vartype,
+    void declDouble(vluint32_t code, const char* name,
+                    int dtypenum, fstVarDir vardir, fstVarType vartype,
                     int arraynum) {
-        declSymbol(code, name, vardir, vartype, arraynum, 2);
+        declSymbol(code, name, dtypenum, vardir, vartype, arraynum, 2);
     }
-    void declFloat(vluint32_t code, const char* name, fstVarDir vardir, fstVarType vartype,
+    void declFloat(vluint32_t code, const char* name,
+                   int dtypenum, fstVarDir vardir, fstVarType vartype,
                    int arraynum) {
-        declSymbol(code, name, vardir, vartype, arraynum, 1);
+        declSymbol(code, name, dtypenum, vardir, vartype, arraynum, 1);
     }
-    void declQuad(vluint32_t code, const char* name, fstVarDir vardir, fstVarType vartype,
+    void declQuad(vluint32_t code, const char* name,
+                  int dtypenum, fstVarDir vardir, fstVarType vartype,
                   int arraynum, int msb, int lsb) {
-        declSymbol(code, name, vardir, vartype, arraynum, msb - lsb + 1);
+        declSymbol(code, name, dtypenum, vardir, vartype, arraynum, msb - lsb + 1);
     }
-    void declArray(vluint32_t code, const char* name, fstVarDir vardir, fstVarType vartype,
+    void declArray(vluint32_t code, const char* name,
+                   int dtypenum, fstVarDir vardir, fstVarType vartype,
                    int arraynum, int msb, int lsb) {
-        declSymbol(code, name, vardir, vartype, arraynum, msb - lsb + 1);
+        declSymbol(code, name, dtypenum, vardir, vartype, arraynum, msb - lsb + 1);
     }
 
     /// Inside dumping routines, dump one signal if it has changed
@@ -144,12 +157,18 @@ public:
         fstWriterEmitValueChange(m_fst, m_code2symbol[code], array2Str(newval, bits));
     }
 
-    void fullBit(vluint32_t code, const vluint32_t newval) { chgBit(code, newval); }
-    void fullBus(vluint32_t code, const vluint32_t newval, int bits) { chgBus(code, newval, bits); }
-    void fullDouble(vluint32_t code, const double newval) { chgDouble(code, newval); }
-    void fullFloat(vluint32_t code, const float newval) { chgFloat(code, newval); }
-    void fullQuad(vluint32_t code, const vluint64_t newval, int bits) { chgQuad(code, newval, bits); }
-    void fullArray(vluint32_t code, const vluint32_t* newval, int bits) { chgArray(code, newval, bits); }
+    void fullBit(vluint32_t code, const vluint32_t newval) {
+        chgBit(code, newval); }
+    void fullBus(vluint32_t code, const vluint32_t newval, int bits) {
+        chgBus(code, newval, bits); }
+    void fullDouble(vluint32_t code, const double newval) {
+        chgDouble(code, newval); }
+    void fullFloat(vluint32_t code, const float newval) {
+        chgFloat(code, newval); }
+    void fullQuad(vluint32_t code, const vluint64_t newval, int bits) {
+        chgQuad(code, newval, bits); }
+    void fullArray(vluint32_t code, const vluint32_t* newval, int bits) {
+        chgArray(code, newval, bits); }
 
     void declTriBit   (vluint32_t code, const char* name, int arraynum);
     void declTriBus   (vluint32_t code, const char* name, int arraynum, int msb, int lsb);
@@ -203,11 +222,11 @@ public:
     void dump(int timestamp) { dump(static_cast<vluint64_t>(timestamp)); }
     /// Set time units (s/ms, defaults to ns)
     /// See also VL_TIME_PRECISION, and VL_TIME_MULTIPLIER in verilated.h
-    void set_time_unit(const char* unit) { /* TODO */ }
+    void set_time_unit(const char* unitp) { m_sptrace.set_time_unit(unitp); }
     void set_time_unit(const std::string& unit) { set_time_unit(unit.c_str()); }
     /// Set time resolution (s/ms, defaults to ns)
     /// See also VL_TIME_PRECISION, and VL_TIME_MULTIPLIER in verilated.h
-    void set_time_resolution(const char* unit) { /* TODO */ }
+    void set_time_resolution(const char* unitp) { m_sptrace.set_time_resolution(unitp); }
     void set_time_resolution(const std::string& unit) { set_time_resolution(unit.c_str()); }
 
     /// Internal class access
